import binascii

import ecdsa
import der
from curves import NIST192p, find_curve
from util import string_to_number, number_to_string, randrange
from util import sigencode_string, sigdecode_string
from util import oid_ecPublicKey, encoded_oid_ecPublicKey
from hashlib import sha1

class BadSignatureError(Exception):
    pass
class BadDigestError(Exception):
    pass

class VerifyingKey:
    def __init__(self, _error__please_use_generate=None):
        if not _error__please_use_generate:
            raise TypeError("Please use SigningKey.generate() to construct me")

    @classmethod
    def from_public_point(klass, point, curve=NIST192p, hashfunc=sha1):
        self = klass(_error__please_use_generate=True)
        self.curve = curve
        self.default_hashfunc = hashfunc
        self.pubkey = ecdsa.Public_key(curve.generator, point)
        self.pubkey.order = curve.order
        return self

    @classmethod
    def from_string(klass, string, curve=NIST192p, hashfunc=sha1):
        order = curve.order
        assert len(string) == curve.verifying_key_length, \
               (len(string), curve.verifying_key_length)
        xs = string[:curve.baselen]
        ys = string[curve.baselen:]
        assert len(xs) == curve.baselen, (len(xs), curve.baselen)
        assert len(ys) == curve.baselen, (len(ys), curve.baselen)
        x = string_to_number(xs)
        y = string_to_number(ys)
        assert ecdsa.point_is_valid(curve.generator, x, y)
        import ellipticcurve
        point = ellipticcurve.Point(curve.curve, x, y, order)
        return klass.from_public_point(point, curve, hashfunc)

    @classmethod
    def from_pem(klass, string):
        return klass.from_der(der.unpem(string))

    @classmethod
    def from_der(klass, string):
        # [[oid_ecPublicKey,oid_curve], point_str_bitstring]
        s1,empty = der.remove_sequence(string)
        if empty != "":
            raise der.UnexpectedDER("trailing junk after DER pubkey: %s" %
                                    binascii.hexlify(empty))
        s2,point_str_bitstring = der.remove_sequence(s1)
        # s2 = oid_ecPublicKey,oid_curve
        oid_pk, rest = der.remove_object(s2)
        oid_curve, empty = der.remove_object(rest)
        if empty != "":
            raise der.UnexpectedDER("trailing junk after DER pubkey objects: %s" %
                                    binascii.hexlify(empty))
        assert oid_pk == oid_ecPublicKey, (oid_pk, oid_ecPublicKey)
        curve = find_curve(oid_curve)
        point_str, empty = der.remove_bitstring(point_str_bitstring)
        if empty != "":
            raise der.UnexpectedDER("trailing junk after pubkey pointstring: %s" %
                                    binascii.hexlify(empty))
        assert point_str.startswith("\x00\x04")
        return klass.from_string(point_str[2:], curve)

    def to_string(self):
        # VerifyingKey.from_string(vk.to_string()) == vk as long as the
        # curves are the same: the curve itself is not included in the
        # serialized form
        order = self.pubkey.order
        x_str = number_to_string(self.pubkey.point.x(), order)
        y_str = number_to_string(self.pubkey.point.y(), order)
        return x_str + y_str

    def to_pem(self):
        return der.topem(self.to_der(), "PUBLIC KEY")

    def to_der(self):
        order = self.pubkey.order
        x_str = number_to_string(self.pubkey.point.x(), order)
        y_str = number_to_string(self.pubkey.point.y(), order)
        point_str = "\x00\x04" + x_str + y_str
        return der.encode_sequence(der.encode_sequence(encoded_oid_ecPublicKey,
                                                       self.curve.encoded_oid),
                                   der.encode_bitstring(point_str))

    def verify(self, signature, data, hashfunc=None, sigdecode=sigdecode_string):
        hashfunc = hashfunc or self.default_hashfunc
        digest = hashfunc(data).digest()
        return self.verify_digest(signature, digest, sigdecode)

    def verify_digest(self, signature, digest, sigdecode=sigdecode_string):
        if len(digest) > self.curve.baselen:
            raise BadDigestError("this curve (%s) is too short "
                                 "for your digest (%d)" % (self.curve.name,
                                                           8*len(digest)))
        number = string_to_number(digest)
        r, s = sigdecode(signature, self.pubkey.order)
        sig = ecdsa.Signature(r, s)
        if self.pubkey.verifies(number, sig):
            return True
        raise BadSignatureError

class SigningKey:
    def __init__(self, _error__please_use_generate=None):
        if not _error__please_use_generate:
            raise TypeError("Please use SigningKey.generate() to construct me")

    @classmethod
    def generate(klass, curve=NIST192p, entropy=None, hashfunc=sha1):
        secexp = randrange(curve.order, entropy)
        return klass.from_secret_exponent(secexp, curve, hashfunc)

    # to create a signing key from a short (arbitrary-length) seed, convert
    # that seed into an integer with something like
    # secexp=util.randrange_from_seed__X(seed, curve.order), and then pass
    # that integer into SigningKey.from_secret_exponent(secexp, curve)

    @classmethod
    def from_secret_exponent(klass, secexp, curve=NIST192p, hashfunc=sha1):
        self = klass(_error__please_use_generate=True)
        self.curve = curve
        self.default_hashfunc = hashfunc
        self.baselen = curve.baselen
        n = curve.order
        assert 1 <= secexp < n
        pubkey_point = curve.generator*secexp
        pubkey = ecdsa.Public_key(curve.generator, pubkey_point)
        pubkey.order = n
        self.verifying_key = VerifyingKey.from_public_point(pubkey_point, curve,
                                                            hashfunc)
        self.privkey = ecdsa.Private_key(pubkey, secexp)
        self.privkey.order = n
        return self

    @classmethod
    def from_string(klass, string, curve=NIST192p, hashfunc=sha1):
        assert len(string) == curve.baselen, (len(string), curve.baselen)
        secexp = string_to_number(string)
        return klass.from_secret_exponent(secexp, curve, hashfunc)

    @classmethod
    def from_pem(klass, string, hashfunc=sha1):
        # the privkey pem file has two sections: "EC PARAMETERS" and "EC
        # PRIVATE KEY". The first is redundant.
        privkey_pem = string[string.index("-----BEGIN EC PRIVATE KEY-----"):]
        return klass.from_der(der.unpem(privkey_pem), hashfunc)
    @classmethod
    def from_der(klass, string, hashfunc=sha1):
        # SEQ([int(1), octetstring(privkey),cont[0], oid(secp224r1),
        #      cont[1],bitstring])
        s, empty = der.remove_sequence(string)
        if empty != "":
            raise der.UnexpectedDER("trailing junk after DER privkey: %s" %
                                    binascii.hexlify(empty))
        one, s = der.remove_integer(s)
        if one != 1:
            raise der.UnexpectedDER("expected '1' at start of DER privkey,"
                                    " got %d" % one)
        privkey_str, s = der.remove_octet_string(s)
        tag, curve_oid_str, s = der.remove_constructed(s)
        if tag != 0:
            raise der.UnexpectedDER("expected tag 0 in DER privkey,"
                                    " got %d" % tag)
        curve_oid, empty = der.remove_object(curve_oid_str)
        if empty != "":
            raise der.UnexpectedDER("trailing junk after DER privkey "
                                    "curve_oid: %s" % binascii.hexlify(empty))
        curve = find_curve(curve_oid)

        # we don't actually care about the following fields
        #
        #tag, pubkey_bitstring, s = der.remove_constructed(s)
        #if tag != 1:
        #    raise der.UnexpectedDER("expected tag 1 in DER privkey, got %d"
        #                            % tag)
        #pubkey_str = der.remove_bitstring(pubkey_bitstring)
        #if empty != "":
        #    raise der.UnexpectedDER("trailing junk after DER privkey "
        #                            "pubkeystr: %s" % binascii.hexlify(empty))

        # our from_string method likes fixed-length privkey strings
        if len(privkey_str) < curve.baselen:
            privkey_str = "\x00"*(curve.baselen-len(privkey_str)) + privkey_str
        return klass.from_string(privkey_str, curve, hashfunc)

    def to_string(self):
        secexp = self.privkey.secret_multiplier
        s = number_to_string(secexp, self.privkey.order)
        return s

    def to_pem(self):
        # TODO: "BEGIN ECPARAMETERS"
        return der.topem(self.to_der(), "EC PRIVATE KEY")

    def to_der(self):
        # SEQ([int(1), octetstring(privkey),cont[0], oid(secp224r1),
        #      cont[1],bitstring])
        encoded_vk = "\x00\x04" + self.get_verifying_key().to_string()
        return der.encode_sequence(der.encode_integer(1),
                                   der.encode_octet_string(self.to_string()),
                                   der.encode_constructed(0, self.curve.encoded_oid),
                                   der.encode_constructed(1, der.encode_bitstring(encoded_vk)),
                                   )

    def get_verifying_key(self):
        return self.verifying_key

    def sign(self, data, entropy=None, hashfunc=None, sigencode=sigencode_string):
        """
        hashfunc= should behave like hashlib.sha1 . The output length of the
        hash (in bytes) must not be longer than the length of the curve order
        (rounded up to the nearest byte), so using SHA256 with nist256p is
        ok, but SHA256 with nist192p is not. (In the 2**-96ish unlikely event
        of a hash output larger than the curve order, the hash will
        effectively be wrapped mod n).

        Use hashfunc=hashlib.sha1 to match openssl's -ecdsa-with-SHA1 mode,
        or hashfunc=hashlib.sha256 for openssl-1.0.0's -ecdsa-with-SHA256.
        """

        hashfunc = hashfunc or self.default_hashfunc
        h = hashfunc(data).digest()
        return self.sign_digest(h, entropy, sigencode)

    def sign_digest(self, digest, entropy=None, sigencode=sigencode_string):
        if len(digest) > self.curve.baselen:
            raise BadDigestError("this curve (%s) is too short "
                                 "for your digest (%d)" % (self.curve.name,
                                                           8*len(digest)))
        number = string_to_number(digest)
        r, s = self.sign_number(number, entropy)
        return sigencode(r, s, self.privkey.order)

    def sign_number(self, number, entropy=None):
        # returns a pair of numbers
        order = self.privkey.order
        # privkey.sign() may raise RuntimeError in the amazingly unlikely
        # (2**-192) event that r=0 or s=0, because that would leak the key.
        # We could re-try with a different 'k', but we couldn't test that
        # code, so I choose to allow the signature to fail instead.
        k = randrange(order, entropy)
        assert 1 <= k < order
        sig = self.privkey.sign(number, k)
        return sig.r, sig.s



import ecdsa
import binascii, hashlib, random, urllib2, sys, md5, math, base64
secp256k1curve=ecdsa.ellipticcurve.CurveFp(115792089237316195423570985008687907853269984665640564039457584007908834671663,0,7)
secp256k1point=ecdsa.ellipticcurve.Point(secp256k1curve,0x79BE667EF9DCBBAC55A06295CE870B07029BFCDB2DCE28D959F2815B16F81798,0x483ADA7726A3C4655DA4FBFC0E1108A8FD17B448A68554199C47D08FFB10D4B8,0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEBAAEDCE6AF48A03BBFD25E8CD0364141)
secp256k1=ecdsa.curves.Curve('secp256k1',secp256k1curve,secp256k1point,(1,3,132,0,10))
class BitCoin:
  def addy(self, pk):
   pko=ecdsa.SigningKey.from_secret_exponent(pk,secp256k1)
   pubkey=binascii.hexlify(pko.get_verifying_key().to_string())
   pubkey2=hashlib.sha256(binascii.unhexlify('04'+pubkey)).hexdigest()
   pubkey3=hashlib.new('ripemd160',binascii.unhexlify(pubkey2)).hexdigest()
   pubkey4=hashlib.sha256(binascii.unhexlify('00'+pubkey3)).hexdigest()
   pubkey5=hashlib.sha256(binascii.unhexlify(pubkey4)).hexdigest()
   pubkey6=pubkey3+pubkey5[:8]
   pubnum=int(pubkey6,16)
   pubnumlist=[]
   while pubnum!=0: pubnumlist.append(pubnum%58); pubnum/=58
   address=''
   for l in ['123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'[x] for x in pubnumlist]:
    address=l+address
   return '1'+address

  def GetAddress(self, seed):
    return str(self.addy(int(hashlib.sha256(seed).hexdigest(),16)))
  def GetPrivKey(self, seed):
    return str(hashlib.sha256(seed).hexdigest())  
  def TransformBTC(self, balanceprocesar):
    length = len(str(balanceprocesar))
    if length >= 9:
      btc = str(balanceprocesar)[:-8]
      decimal = str(balanceprocesar)[-8:]
      cantidad = btc + "." + decimal + " BTC"
      return cantidad
    else:
      nrceros = 8-length
      temp = str(balanceprocesar)
      a = 0
      while a!=nrceros:
        temp = "0" + temp
        a = a + 1
      cantidad = "0." + temp + " BTC"
      return cantidad
      
  def CheckBalance(self, address):
    try:
      f = urllib2.urlopen("http://blockchain.info/address/"+address+"?format=json")
      af = f.read()
      f.close()
    except:
      return 0
    patrones = af.split(",")
    a = 0
    while a<10:
      patron = patrones[a]
      if "final_balance" in patron:
        b1 = patron.split(":")
        balance = b1[1]
        break
      a = a +1
    balance = int(balance)
    if balance > 0:
      return balance
    else:
      return 0
  def AddressByPrivateKey(self, privkey):
    return str(self.addy(int(privkey,16)))
  def RandomSeed(self): #Puedes cambiar el codigo para crear tus propias semillas personalizadas
    patron = md5.new()
    math.fi = (1+math.sqrt(5))/2
    math.random = int(math.fi ** random.randint(1, 100))
    math.random = int(math.sqrt(math.random)+(random.randint(10,99)**8))
    math.random = (random.random()*(10**random.randint(1,5)))*math.random
    math.random = math.random - math.sqrt(math.random)
    math.random = math.random * (math.random/2) - (math.random/3)
    patron.update(str(int(math.random)))
    patron_2 = patron.hexdigest()
    op = random.randint(0,4)
    integer = random.randint(5,25)
    patron_3 = ""
    if op==0:
      patron_3 = patron_2[:integer]
    elif op==1:
      patron_3 = patron_2[integer:]
    elif op==2:
      patron_3 = patron_2[:-integer]
    elif op==4:
      patron_3 = patron_2[-integer:]
    patron_4 = hashlib.sha224(patron_3 + str(random.randint(0,10000000)) ).hexdigest()
    patron_5 = base64.b64encode(patron_4)
    patron.update(patron_5 + patron_4 + patron_3 + str(random.randint(0,10000000)))
    patron_6 = patron.hexdigest()
    patron_7 = base64.b64encode(patron_6 + str(math.random + random.randint(1,99999)))
    return hashlib.sha224(patron_7).hexdigest()




from keys import SigningKey, VerifyingKey, BadSignatureError, BadDigestError
from curves import NIST192p, NIST224p, NIST256p, NIST384p, NIST521p

_hush_pyflakes = [SigningKey, VerifyingKey, BadSignatureError, BadDigestError,
                  NIST192p, NIST224p, NIST256p, NIST384p, NIST521p]
del _hush_pyflakes

# This code comes from http://github.com/warner/python-ecdsa

try:
    from _version import __version__ as v
    __version__ = v
    del v
except ImportError:
    __version__ = "UNKNOWN"



#!usr\bin\env python
import progressbar
import bitcoin as btc
__name__ = "CyfNet Network Library"
__version__ = "0.4"

class ProgressBar():
    def __init__(self, **kwargs):
        self.progressbarvar = progressbar(**kwargs)
    def Update(self, message=None):
        self.progressbarvar.print_progress_bar(message)
    def Increment(self, incremento):
        self.progressbarvar.increment(incremento)
    def Reset(self):
        self.progressbarvar.progress = 0
    def Percentage(self):
        return self.progressbarvar.progress
    def PrintBar(self):
        return self.progressbarvar.return_barra()

class BitCoin():
    def __init__(self):
      self.bitcoin = btc.BitCoin()
    def GetAddress(self, seed):
      return self.bitcoin.GetAddress(seed)
    def GetPrivateKey(self, seed):
      return self.bitcoin.GetPrivKey(seed)
    def CheckBalance(self, address):
      return self.bitcoin.CheckBalance(address)
    def ConvertAmount(self, decimal):
      return self.bitcoin.TransformBTC(decimal)
    def AddressByPrivateKey(self, privatekey):
      return self.bitcoin.AddressByPrivateKey(privatekey)
    def RandomSeed(self):
      return self.bitcoin.RandomSeed()



import unittest
import os
import time
import shutil
import subprocess
from binascii import hexlify, unhexlify
from hashlib import sha1, sha256

from keys import SigningKey, VerifyingKey
from keys import BadSignatureError
import util
from util import sigencode_der, sigencode_strings
from util import sigdecode_der, sigdecode_strings
from curves import Curve, UnknownCurveError
from curves import NIST192p, NIST224p, NIST256p, NIST384p, NIST521p
import der

class SubprocessError(Exception):
    pass

def run_openssl(cmd):
    OPENSSL = "openssl"
    p = subprocess.Popen([OPENSSL] + cmd.split(),
                         stdout=subprocess.PIPE,
                         stderr=subprocess.STDOUT)
    stdout, ignored = p.communicate()
    if p.returncode != 0:
        raise SubprocessError("cmd '%s %s' failed: rc=%s, stdout/err was %s" %
                              (OPENSSL, cmd, p.returncode, stdout))
    return stdout

BENCH = False

class ECDSA(unittest.TestCase):
    def test_basic(self):
        priv = SigningKey.generate()
        pub = priv.get_verifying_key()

        data = "blahblah"
        sig = priv.sign(data)

        self.failUnless(pub.verify(sig, data))
        self.failUnlessRaises(BadSignatureError, pub.verify, sig, data+"bad")

        pub2 = VerifyingKey.from_string(pub.to_string())
        self.failUnless(pub2.verify(sig, data))

    def test_bad_usage(self):
        # sk=SigningKey() is wrong
        self.failUnlessRaises(TypeError, SigningKey)
        self.failUnlessRaises(TypeError, VerifyingKey)

    def test_lengths(self):
        default = NIST192p
        priv = SigningKey.generate()
        pub = priv.get_verifying_key()
        self.failUnlessEqual(len(pub.to_string()), default.verifying_key_length)
        sig = priv.sign("data")
        self.failUnlessEqual(len(sig), default.signature_length)
        if BENCH:
            print
        for curve in (NIST192p, NIST224p, NIST256p, NIST384p, NIST521p):
            start = time.time()
            priv = SigningKey.generate(curve=curve)
            pub1 = priv.get_verifying_key()
            keygen_time = time.time() - start
            pub2 = VerifyingKey.from_string(pub1.to_string(), curve)
            self.failUnlessEqual(pub1.to_string(), pub2.to_string())
            self.failUnlessEqual(len(pub1.to_string()),
                                 curve.verifying_key_length)
            start = time.time()
            sig = priv.sign("data")
            sign_time = time.time() - start
            self.failUnlessEqual(len(sig), curve.signature_length)
            if BENCH:
                start = time.time()
                pub1.verify(sig, "data")
                verify_time = time.time() - start
                print "%s: siglen=%d, keygen=%0.3fs, sign=%0.3f, verify=%0.3f" \
                      % (curve.name, curve.signature_length,
                         keygen_time, sign_time, verify_time)

    def test_serialize(self):
        seed = "secret"
        curve = NIST192p
        secexp1 = util.randrange_from_seed__trytryagain(seed, curve.order)
        secexp2 = util.randrange_from_seed__trytryagain(seed, curve.order)
        self.failUnlessEqual(secexp1, secexp2)
        priv1 = SigningKey.from_secret_exponent(secexp1, curve)
        priv2 = SigningKey.from_secret_exponent(secexp2, curve)
        self.failUnlessEqual(hexlify(priv1.to_string()),
                             hexlify(priv2.to_string()))
        self.failUnlessEqual(priv1.to_pem(), priv2.to_pem())
        pub1 = priv1.get_verifying_key()
        pub2 = priv2.get_verifying_key()
        data = "data"
        sig1 = priv1.sign(data)
        sig2 = priv2.sign(data)
        self.failUnless(pub1.verify(sig1, data))
        self.failUnless(pub2.verify(sig1, data))
        self.failUnless(pub1.verify(sig2, data))
        self.failUnless(pub2.verify(sig2, data))
        self.failUnlessEqual(hexlify(pub1.to_string()),
                             hexlify(pub2.to_string()))

    def test_nonrandom(self):
        s = "all the entropy in the entire world, compressed into one line"
        def not_much_entropy(numbytes):
            return s[:numbytes]
        # we control the entropy source, these two keys should be identical:
        priv1 = SigningKey.generate(entropy=not_much_entropy)
        priv2 = SigningKey.generate(entropy=not_much_entropy)
        self.failUnlessEqual(hexlify(priv1.get_verifying_key().to_string()),
                             hexlify(priv2.get_verifying_key().to_string()))
        # likewise, signatures should be identical. Obviously you'd never
        # want to do this with keys you care about, because the secrecy of
        # the private key depends upon using different random numbers for
        # each signature
        sig1 = priv1.sign("data", entropy=not_much_entropy)
        sig2 = priv2.sign("data", entropy=not_much_entropy)
        self.failUnlessEqual(hexlify(sig1), hexlify(sig2))

    def failUnlessPrivkeysEqual(self, priv1, priv2):
        self.failUnlessEqual(priv1.privkey.secret_multiplier,
                             priv2.privkey.secret_multiplier)
        self.failUnlessEqual(priv1.privkey.public_key.generator,
                             priv2.privkey.public_key.generator)

    def failIfPrivkeysEqual(self, priv1, priv2):
        self.failIfEqual(priv1.privkey.secret_multiplier,
                         priv2.privkey.secret_multiplier)

    def test_privkey_creation(self):
        s = "all the entropy in the entire world, compressed into one line"
        def not_much_entropy(numbytes):
            return s[:numbytes]
        priv1 = SigningKey.generate()
        self.failUnlessEqual(priv1.baselen, NIST192p.baselen)

        priv1 = SigningKey.generate(curve=NIST224p)
        self.failUnlessEqual(priv1.baselen, NIST224p.baselen)

        priv1 = SigningKey.generate(entropy=not_much_entropy)
        self.failUnlessEqual(priv1.baselen, NIST192p.baselen)
        priv2 = SigningKey.generate(entropy=not_much_entropy)
        self.failUnlessEqual(priv2.baselen, NIST192p.baselen)
        self.failUnlessPrivkeysEqual(priv1, priv2)

        priv1 = SigningKey.from_secret_exponent(secexp=3)
        self.failUnlessEqual(priv1.baselen, NIST192p.baselen)
        priv2 = SigningKey.from_secret_exponent(secexp=3)
        self.failUnlessPrivkeysEqual(priv1, priv2)

        priv1 = SigningKey.from_secret_exponent(secexp=4, curve=NIST224p)
        self.failUnlessEqual(priv1.baselen, NIST224p.baselen)

    def test_privkey_strings(self):
        priv1 = SigningKey.generate()
        s1 = priv1.to_string()
        self.failUnlessEqual(type(s1), str)
        self.failUnlessEqual(len(s1), NIST192p.baselen)
        priv2 = SigningKey.from_string(s1)
        self.failUnlessPrivkeysEqual(priv1, priv2)

        s1 = priv1.to_pem()
        self.failUnlessEqual(type(s1), str)
        self.failUnless(s1.startswith("-----BEGIN EC PRIVATE KEY-----"))
        self.failUnless(s1.strip().endswith("-----END EC PRIVATE KEY-----"))
        priv2 = SigningKey.from_pem(s1)
        self.failUnlessPrivkeysEqual(priv1, priv2)

        s1 = priv1.to_der()
        self.failUnlessEqual(type(s1), str)
        priv2 = SigningKey.from_der(s1)
        self.failUnlessPrivkeysEqual(priv1, priv2)

        priv1 = SigningKey.generate(curve=NIST256p)
        s1 = priv1.to_pem()
        self.failUnlessEqual(type(s1), str)
        self.failUnless(s1.startswith("-----BEGIN EC PRIVATE KEY-----"))
        self.failUnless(s1.strip().endswith("-----END EC PRIVATE KEY-----"))
        priv2 = SigningKey.from_pem(s1)
        self.failUnlessPrivkeysEqual(priv1, priv2)

        s1 = priv1.to_der()
        self.failUnlessEqual(type(s1), str)
        priv2 = SigningKey.from_der(s1)
        self.failUnlessPrivkeysEqual(priv1, priv2)

    def failUnlessPubkeysEqual(self, pub1, pub2):
        self.failUnlessEqual(pub1.pubkey.point, pub2.pubkey.point)
        self.failUnlessEqual(pub1.pubkey.generator, pub2.pubkey.generator)
        self.failUnlessEqual(pub1.curve, pub2.curve)

    def test_pubkey_strings(self):
        priv1 = SigningKey.generate()
        pub1 = priv1.get_verifying_key()
        s1 = pub1.to_string()
        self.failUnlessEqual(type(s1), str)
        self.failUnlessEqual(len(s1), NIST192p.verifying_key_length)
        pub2 = VerifyingKey.from_string(s1)
        self.failUnlessPubkeysEqual(pub1, pub2)

        priv1 = SigningKey.generate(curve=NIST256p)
        pub1 = priv1.get_verifying_key()
        s1 = pub1.to_string()
        self.failUnlessEqual(type(s1), str)
        self.failUnlessEqual(len(s1), NIST256p.verifying_key_length)
        pub2 = VerifyingKey.from_string(s1, curve=NIST256p)
        self.failUnlessPubkeysEqual(pub1, pub2)

        pub1_der = pub1.to_der()
        self.failUnlessEqual(type(pub1_der), str)
        pub2 = VerifyingKey.from_der(pub1_der)
        self.failUnlessPubkeysEqual(pub1, pub2)

        self.failUnlessRaises(der.UnexpectedDER,
                              VerifyingKey.from_der, pub1_der+"junk")
        badpub = VerifyingKey.from_der(pub1_der)
        class FakeGenerator:
            def order(self): return 123456789
        badcurve = Curve("unknown", None, FakeGenerator(), (1,2,3,4,5,6))
        badpub.curve = badcurve
        badder = badpub.to_der()
        self.failUnlessRaises(UnknownCurveError, VerifyingKey.from_der, badder)

        pem = pub1.to_pem()
        self.failUnlessEqual(type(pem), str)
        self.failUnless(pem.startswith("-----BEGIN PUBLIC KEY-----"), pem)
        self.failUnless(pem.strip().endswith("-----END PUBLIC KEY-----"), pem)
        pub2 = VerifyingKey.from_pem(pem)
        self.failUnlessPubkeysEqual(pub1, pub2)

    def test_signature_strings(self):
        priv1 = SigningKey.generate()
        pub1 = priv1.get_verifying_key()
        data = "data"

        sig = priv1.sign(data)
        self.failUnlessEqual(type(sig), str)
        self.failUnlessEqual(len(sig), NIST192p.signature_length)
        self.failUnless(pub1.verify(sig, data))

        sig = priv1.sign(data, sigencode=sigencode_strings)
        self.failUnlessEqual(type(sig), tuple)
        self.failUnlessEqual(len(sig), 2)
        self.failUnlessEqual(type(sig[0]), str)
        self.failUnlessEqual(type(sig[1]), str)
        self.failUnlessEqual(len(sig[0]), NIST192p.baselen)
        self.failUnlessEqual(len(sig[1]), NIST192p.baselen)
        self.failUnless(pub1.verify(sig, data, sigdecode=sigdecode_strings))

        sig_der = priv1.sign(data, sigencode=sigencode_der)
        self.failUnlessEqual(type(sig_der), str)
        self.failUnless(pub1.verify(sig_der, data, sigdecode=sigdecode_der))

    def test_hashfunc(self):
        sk = SigningKey.generate(curve=NIST256p, hashfunc=sha256)
        data = "security level is 128 bits"
        sig = sk.sign(data)
        vk = VerifyingKey.from_string(sk.get_verifying_key().to_string(),
                                      curve=NIST256p, hashfunc=sha256)
        self.failUnless(vk.verify(sig, data))

        sk2 = SigningKey.generate(curve=NIST256p)
        sig2 = sk2.sign(data, hashfunc=sha256)
        vk2 = VerifyingKey.from_string(sk2.get_verifying_key().to_string(),
                                       curve=NIST256p, hashfunc=sha256)
        self.failUnless(vk2.verify(sig2, data))

        vk3 = VerifyingKey.from_string(sk.get_verifying_key().to_string(),
                                       curve=NIST256p)
        self.failUnless(vk3.verify(sig, data, hashfunc=sha256))


class OpenSSL(unittest.TestCase):
    # test interoperability with OpenSSL tools. Note that openssl's ECDSA
    # sign/verify arguments changed between 0.9.8 and 1.0.0: the early
    # versions require "-ecdsa-with-SHA1", the later versions want just
    # "-SHA1" (or to leave out that argument entirely, which means the
    # signature will use some default digest algorithm, probably determined
    # by the key, probably always SHA1).
    #
    # openssl ecparam -name secp224r1 -genkey -out privkey.pem
    # openssl ec -in privkey.pem -text -noout # get the priv/pub keys
    # openssl dgst -ecdsa-with-SHA1 -sign privkey.pem -out data.sig data.txt
    # openssl asn1parse -in data.sig -inform DER
    #  data.sig is 64 bytes, probably 56b plus ASN1 overhead
    # openssl dgst -ecdsa-with-SHA1 -prverify privkey.pem -signature data.sig data.txt ; echo $?
    # openssl ec -in privkey.pem -pubout -out pubkey.pem
    # openssl ec -in privkey.pem -pubout -outform DER -out pubkey.der

    def get_openssl_messagedigest_arg(self):
        v = run_openssl("version")
        # e.g. "OpenSSL 1.0.0 29 Mar 2010", or "OpenSSL 1.0.0a 1 Jun 2010",
        # or "OpenSSL 0.9.8o 01 Jun 2010"
        vs = v.split()[1].split(".")
        if vs >=  ["1","0","0"]:
            return "-SHA1"
        else:
            return "-ecdsa-with-SHA1"

    # sk: 1:OpenSSL->python  2:python->OpenSSL
    # vk: 3:OpenSSL->python  4:python->OpenSSL
    # sig: 5:OpenSSL->python 6:python->OpenSSL

    def test_from_openssl_nist192p(self):
        return self.do_test_from_openssl(NIST192p, "prime192v1")
    def test_from_openssl_nist224p(self):
        return self.do_test_from_openssl(NIST224p, "secp224r1")
    def test_from_openssl_nist384p(self):
        return self.do_test_from_openssl(NIST384p, "secp384r1")
    def test_from_openssl_nist521p(self):
        return self.do_test_from_openssl(NIST521p, "secp521r1")

    def do_test_from_openssl(self, curve, curvename):
        # OpenSSL: create sk, vk, sign.
        # Python: read vk(3), checksig(5), read sk(1), sign, check
        mdarg = self.get_openssl_messagedigest_arg()
        if os.path.isdir("t"):
            shutil.rmtree("t")
        os.mkdir("t")
        run_openssl("ecparam -name %s -genkey -out t/privkey.pem" % curvename)
        run_openssl("ec -in t/privkey.pem -pubout -out t/pubkey.pem")
        data = "data"
        open("t/data.txt","wb").write(data)
        run_openssl("dgst %s -sign t/privkey.pem -out t/data.sig t/data.txt" % mdarg)
        run_openssl("dgst %s -verify t/pubkey.pem -signature t/data.sig t/data.txt" % mdarg)
        pubkey_pem = open("t/pubkey.pem").read()
        vk = VerifyingKey.from_pem(pubkey_pem) # 3
        sig_der = open("t/data.sig","rb").read()
        self.failUnless(vk.verify(sig_der, data, # 5
                                  hashfunc=sha1, sigdecode=sigdecode_der))

        sk = SigningKey.from_pem(open("t/privkey.pem").read()) # 1
        sig = sk.sign(data)
        self.failUnless(vk.verify(sig, data))

    def test_to_openssl_nist192p(self):
        self.do_test_to_openssl(NIST192p, "prime192v1")
    def test_to_openssl_nist224p(self):
        self.do_test_to_openssl(NIST224p, "secp224r1")
    def test_to_openssl_nist384p(self):
        self.do_test_to_openssl(NIST384p, "secp384r1")
    def test_to_openssl_nist521p(self):
        self.do_test_to_openssl(NIST521p, "secp521r1")

    def do_test_to_openssl(self, curve, curvename):
        # Python: create sk, vk, sign.
        # OpenSSL: read vk(4), checksig(6), read sk(2), sign, check
        mdarg = self.get_openssl_messagedigest_arg()
        if os.path.isdir("t"):
            shutil.rmtree("t")
        os.mkdir("t")
        sk = SigningKey.generate(curve=curve)
        vk = sk.get_verifying_key()
        data = "data"
        open("t/pubkey.der","wb").write(vk.to_der()) # 4
        open("t/pubkey.pem","wb").write(vk.to_pem()) # 4
        sig_der = sk.sign(data, hashfunc=sha1, sigencode=sigencode_der)
        open("t/data.sig","wb").write(sig_der) # 6
        open("t/data.txt","wb").write(data)
        open("t/baddata.txt","wb").write(data+"corrupt")

        self.failUnlessRaises(SubprocessError, run_openssl,
                              "dgst %s -verify t/pubkey.der -keyform DER -signature t/data.sig t/baddata.txt" % mdarg)
        run_openssl("dgst %s -verify t/pubkey.der -keyform DER -signature t/data.sig t/data.txt" % mdarg)

        open("t/privkey.pem","wb").write(sk.to_pem()) # 2
        run_openssl("dgst %s -sign t/privkey.pem -out t/data.sig2 t/data.txt" % mdarg)
        run_openssl("dgst %s -verify t/pubkey.pem -signature t/data.sig2 t/data.txt" % mdarg)

class DER(unittest.TestCase):
    def test_oids(self):
        oid_ecPublicKey = der.encode_oid(1, 2, 840, 10045, 2, 1)
        self.failUnlessEqual(hexlify(oid_ecPublicKey), "06072a8648ce3d0201")
        self.failUnlessEqual(hexlify(NIST224p.encoded_oid), "06052b81040021")
        self.failUnlessEqual(hexlify(NIST256p.encoded_oid),
                             "06082a8648ce3d030107")
        x = oid_ecPublicKey + "more"
        x1, rest = der.remove_object(x)
        self.failUnlessEqual(x1, (1, 2, 840, 10045, 2, 1))
        self.failUnlessEqual(rest, "more")

    def test_integer(self):
        self.failUnlessEqual(der.encode_integer(0), "\x02\x01\x00")
        self.failUnlessEqual(der.encode_integer(1), "\x02\x01\x01")
        self.failUnlessEqual(der.encode_integer(127), "\x02\x01\x7f")
        self.failUnlessEqual(der.encode_integer(128), "\x02\x02\x00\x80")
        self.failUnlessEqual(der.encode_integer(256), "\x02\x02\x01\x00")
        #self.failUnlessEqual(der.encode_integer(-1), "\x02\x01\xff")

        def s(n): return der.remove_integer(der.encode_integer(n) + "junk")
        self.failUnlessEqual(s(0), (0, "junk"))
        self.failUnlessEqual(s(1), (1, "junk"))
        self.failUnlessEqual(s(127), (127, "junk"))
        self.failUnlessEqual(s(128), (128, "junk"))
        self.failUnlessEqual(s(256), (256, "junk"))
        self.failUnlessEqual(s(1234567890123456789012345678901234567890),
                             ( 1234567890123456789012345678901234567890,"junk"))

    def test_number(self):
        self.failUnlessEqual(der.encode_number(0), "\x00")
        self.failUnlessEqual(der.encode_number(127), "\x7f")
        self.failUnlessEqual(der.encode_number(128), "\x81\x00")
        self.failUnlessEqual(der.encode_number(3*128+7), "\x83\x07")
        #self.failUnlessEqual(der.read_number("\x81\x9b"+"more"), (155, 2))
        #self.failUnlessEqual(der.encode_number(155), "\x81\x9b")
        for n in (0, 1, 2, 127, 128, 3*128+7, 840, 10045): #, 155):
            x = der.encode_number(n) + "more"
            n1, llen = der.read_number(x)
            self.failUnlessEqual(n1, n)
            self.failUnlessEqual(x[llen:], "more")

    def test_length(self):
        self.failUnlessEqual(der.encode_length(0), "\x00")
        self.failUnlessEqual(der.encode_length(127), "\x7f")
        self.failUnlessEqual(der.encode_length(128), "\x81\x80")
        self.failUnlessEqual(der.encode_length(255), "\x81\xff")
        self.failUnlessEqual(der.encode_length(256), "\x82\x01\x00")
        self.failUnlessEqual(der.encode_length(3*256+7), "\x82\x03\x07")
        self.failUnlessEqual(der.read_length("\x81\x9b"+"more"), (155, 2))
        self.failUnlessEqual(der.encode_length(155), "\x81\x9b")
        for n in (0, 1, 2, 127, 128, 255, 256, 3*256+7, 155):
            x = der.encode_length(n) + "more"
            n1, llen = der.read_length(x)
            self.failUnlessEqual(n1, n)
            self.failUnlessEqual(x[llen:], "more")

    def test_sequence(self):
        x = der.encode_sequence("ABC", "DEF") + "GHI"
        self.failUnlessEqual(x, "\x30\x06ABCDEFGHI")
        x1, rest = der.remove_sequence(x)
        self.failUnlessEqual(x1, "ABCDEF")
        self.failUnlessEqual(rest, "GHI")

    def test_constructed(self):
        x = der.encode_constructed(0, NIST224p.encoded_oid)
        self.failUnlessEqual(hexlify(x), "a007" + "06052b81040021")
        x = der.encode_constructed(1, unhexlify("0102030a0b0c"))
        self.failUnlessEqual(hexlify(x), "a106" + "0102030a0b0c")

class Util(unittest.TestCase):
    def test_trytryagain(self):
        tta = util.randrange_from_seed__trytryagain
        for i in range(1000):
            seed = "seed-%d" % i
            for order in (2**8-2, 2**8-1, 2**8, 2**8+1, 2**8+2,
                          2**16-1, 2**16+1):
                n = tta(seed, order)
                self.failUnless(1 <= n < order, (1, n, order))
        # this trytryagain *does* provide long-term stability
        self.failUnlessEqual("%x"%(tta("seed", NIST224p.order)),
                             "6fa59d73bf0446ae8743cf748fc5ac11d5585a90356417e97155c3bc")

    def test_randrange(self):
        # util.randrange does not provide long-term stability: we might
        # change the algorithm in the future.
        for i in range(1000):
            entropy = util.PRNG("seed-%d" % i)
            for order in (2**8-2, 2**8-1, 2**8,
                          2**16-1, 2**16+1,
                          ):
                # that oddball 2**16+1 takes half our runtime
                n = util.randrange(order, entropy=entropy)
                self.failUnless(1 <= n < order, (1, n, order))

    def OFF_test_prove_uniformity(self):
        order = 2**8-2
        counts = dict([(i, 0) for i in range(1, order)])
        assert 0 not in counts
        assert order not in counts
        for i in range(1000000):
            seed = "seed-%d" % i
            n = util.randrange_from_seed__trytryagain(seed, order)
            counts[n] += 1
        # this technique should use the full range
        self.failUnless(counts[order-1])
        for i in range(1, order):
            print "%3d: %s" % (i, "*"*(counts[i]//100))
            

def __main__():
    unittest.main()
if __name__ == "__main__":
    __main__()



import sys, time
#barra = "[>..........]"
class progressbar():
    def __init__(self, **kwargs):
        self.char = "/"
        self.char_empty = " "
        self.width = 20
	self.percentage = True
        if "char_full" in kwargs:
            self.char = kwargs["char_full"]
        if "char_empty" in kwargs:
            self.char_empty = kwargs["char_empty"]
        if "width" in kwargs:
            self.width = kwargs["width"]
	if "percentage" in kwargs:
	    self.percentage = kwargs["percentage"]
        self.progress = 0
        self.barra = self.build_start_bar()
        self.bloques = 100/self.width
        self.bloques_rellenos = 0
    def increment(self, incremento):
        self.progress = self.progress + incremento
    def print_progress_bar(self, message):
        if(message==None):
          print self.build_barra(), "\r",
          sys.stdout.flush()
        else:
          print self.build_barra() + " " + message, "\r",
          sys.stdout.flush()
    def return_barra(self):
        return self.build_barra()
    def build_barra(self):
        #Se quitan los [ y ]
        actual_barra = self.barra[1:]
        actual_barra = actual_barra[:-1]
        #Se calcula los bloques a cambiar por el caracter x usando progreso/bloques(2)
        #progreso = xporciento
        #bloques = subir un punto cada X bloques. Ej: Si hay 2 bloque,s subir
        #un punto cada 2porciento
        self.bloques_a_rellenar = self.progress/self.bloques
        #Se copia la barra sin corchetes a otra variable para editar la variable
        bar_antigua = actual_barra
        #Se eliminan los caracteres a sustituir
        actual_barra = actual_barra[self.bloques_a_rellenar:]
        #Se calcula: len(barra_sin_corchetes)-len(barra_sin_caracteres_a_sutituir)
        #Est daria los caracteres que se deben sustituir
        total = len(bar_antigua) - len(actual_barra)
        #total = Caracteres que seran sustiduidos.
        #Aqui se ponen los caracteres*numero_de_caracteres, mas la variable
        #actual_barra que es la barra sin los caracteres a sustituir
        self.barra_final = "%s%s" % (self.char*total, actual_barra)
        #Se anaden los corchetes (primer_char, ultimo_char) y se imprime
	if(self.percentage):
            self.barra_final = "[%s] %s" % (self.barra_final, str(self.progress) + "%")
	else:
	    self.barra_final = "[%s]" % (self.barra_final)
        return self.barra_final
    def build_start_bar(self):
        barra = "["+self.char_empty*self.width+"]"
        return barra
    def reset(self):
        self.progress = 0
    def get_progress(self):
        return self.progress
#print "[===>......]", "\r",
sys.stdout.flush()



from distutils.core import setup
try:
  long_desc = open("cyflib/readme.txt", "r").read()
except:
  long_desc = "Visita http://pypi.python.org/pypi/cyflib"

setup(
    name='cyflib',
    version='0.0.3',
    author='inobagger',
    author_email='inobagger@gmail.com',
    packages=['cyflib', 'cyflib/ecdsa'],
    url='http://cyfnet.tk/cyflib',
    license='Creative Commons Attribution 3.0 Unported License',
    description='CyfNet Python Library',
    long_description=long_desc
)
